"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
exports.__esModule = true;
exports.useFind = void 0;
var meteor_1 = require("meteor/meteor");
var mongo_1 = require("meteor/mongo");
var react_1 = require("react");
var tracker_1 = require("meteor/tracker");
var useFindReducer = function (data, action) {
    switch (action.type) {
        case 'refresh':
            return action.data;
        case 'addedAt':
            return __spreadArray(__spreadArray(__spreadArray([], data.slice(0, action.atIndex)), [
                action.document
            ]), data.slice(action.atIndex));
        case 'changedAt':
            return __spreadArray(__spreadArray(__spreadArray([], data.slice(0, action.atIndex)), [
                action.document
            ]), data.slice(action.atIndex + 1));
        case 'removedAt':
            return __spreadArray(__spreadArray([], data.slice(0, action.atIndex)), data.slice(action.atIndex + 1));
        case 'movedTo':
            var doc = data[action.fromIndex];
            var copy = __spreadArray(__spreadArray([], data.slice(0, action.fromIndex)), data.slice(action.fromIndex + 1));
            copy.splice(action.toIndex, 0, doc);
            return copy;
    }
};
// Check for valid Cursor or null.
// On client, we should have a Mongo.Cursor (defined in
// https://github.com/meteor/meteor/blob/devel/packages/minimongo/cursor.js and
// https://github.com/meteor/meteor/blob/devel/packages/mongo/collection.js).
// On server, however, we instead get a private Cursor type from
// https://github.com/meteor/meteor/blob/devel/packages/mongo/mongo_driver.js
// which has fields _mongo and _cursorDescription.
var checkCursor = function (cursor) {
    if (cursor !== null && cursor !== undefined && !(cursor instanceof mongo_1.Mongo.Cursor) &&
        !(cursor._mongo && cursor._cursorDescription)) {
        console.warn('Warning: useFind requires an instance of Mongo.Cursor. '
            + 'Make sure you do NOT call .fetch() on your cursor.');
    }
};
// Synchronous data fetch. It uses cursor observing instead of cursor.fetch() because synchronous fetch will be deprecated.
var fetchData = function (cursor) {
    var data = [];
    var observer = cursor.observe({
        addedAt: function (document, atIndex, before) {
            data.splice(atIndex, 0, document);
        }
    });
    observer.stop();
    return data;
};
var useFindClient = function (factory, deps) {
    if (deps === void 0) { deps = []; }
    var cursor = react_1.useMemo(function () {
        // To avoid creating side effects in render, opt out
        // of Tracker integration altogether.
        var cursor = tracker_1.Tracker.nonreactive(factory);
        if (meteor_1.Meteor.isDevelopment) {
            checkCursor(cursor);
        }
        return cursor;
    }, deps);
    var _a = react_1.useReducer(useFindReducer, null, function () {
        if (!(cursor instanceof mongo_1.Mongo.Cursor)) {
            return [];
        }
        return fetchData(cursor);
    }), data = _a[0], dispatch = _a[1];
    // Store information about mounting the component.
    // It will be used to run code only if the component is updated.
    var didMount = react_1.useRef(false);
    react_1.useEffect(function () {
        // Fetch intitial data if cursor was changed.
        if (didMount.current) {
            if (!(cursor instanceof mongo_1.Mongo.Cursor)) {
                return;
            }
            var data_1 = fetchData(cursor);
            dispatch({ type: 'refresh', data: data_1 });
        }
        else {
            didMount.current = true;
        }
        if (!(cursor instanceof mongo_1.Mongo.Cursor)) {
            return;
        }
        var observer = cursor.observe({
            addedAt: function (document, atIndex, before) {
                dispatch({ type: 'addedAt', document: document, atIndex: atIndex });
            },
            changedAt: function (newDocument, oldDocument, atIndex) {
                dispatch({ type: 'changedAt', document: newDocument, atIndex: atIndex });
            },
            removedAt: function (oldDocument, atIndex) {
                dispatch({ type: 'removedAt', atIndex: atIndex });
            },
            movedTo: function (document, fromIndex, toIndex, before) {
                dispatch({ type: 'movedTo', fromIndex: fromIndex, toIndex: toIndex });
            },
            // @ts-ignore
            _suppress_initial: true
        });
        return function () {
            observer.stop();
        };
    }, [cursor]);
    return cursor ? data : cursor;
};
var useFindServer = function (factory, deps) { return (tracker_1.Tracker.nonreactive(function () {
    var _a, _b;
    var cursor = factory();
    if (meteor_1.Meteor.isDevelopment)
        checkCursor(cursor);
    return (_b = (_a = cursor === null || cursor === void 0 ? void 0 : cursor.fetch) === null || _a === void 0 ? void 0 : _a.call(cursor)) !== null && _b !== void 0 ? _b : null;
})); };
exports.useFind = meteor_1.Meteor.isServer
    ? useFindServer
    : useFindClient;
function useFindDev(factory, deps) {
    if (deps === void 0) { deps = []; }
    function warn(expects, pos, arg, type) {
        console.warn("Warning: useFind expected a " + expects + " in it's " + pos + " argument "
            + ("(" + arg + "), but got type of `" + type + "`."));
    }
    if (typeof factory !== 'function') {
        warn("function", "1st", "reactiveFn", factory);
    }
    if (!deps || !Array.isArray(deps)) {
        warn("array", "2nd", "deps", typeof deps);
    }
    return exports.useFind(factory, deps);
}
exports["default"] = meteor_1.Meteor.isDevelopment
    ? useFindDev
    : exports.useFind;
